const Answer = require("../models/Answer");

const { Op } = require("sequelize");
const TheoryTrainAttempts = require("../models/TheoryTrainAttempts");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [],
                }
                : {},
        };
        const info = await Answer.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no Answer!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Answers info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    };
}

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await Answer.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "Answer not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "Answer info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateAnswer = async (req, res) => {
    try {
        const id = req.body.id;

        const theoryTrainAttemptsId = req.body.theoryTrainAttemptsId == undefined ? undefined : req.body.theoryTrainAttemptsId;
        const choiceId = req.body.choiceId == undefined ? undefined : req.body.choiceId;
        const status = req.body.status == undefined ? undefined : req.body.status;

        const result = await Answer.update(
            {
                TheoryTrainAttemptsId: theoryTrainAttemptsId,
                status,
                ChoiceId: choiceId
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `Answer is updated`,
            });
        } else {
            return res.status(204).json({
                success: false,
                message: `Nothing updated for Answer, may be column does not exist`,
            });
        }
    } catch (err) {
        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteAnswerById = async (req, res) => {
    try {
        const id = req.query.id;
        const anwser = await Answer.findByPk(id);

        if (anwser) {
            await anwser.destroy();

            return res.status(200).json({
                success: true,
                message: `Answer with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "Answer does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addAnswer = async (req, res) => {
    try {
        const { theoryTrainAttemptsId, status, choiceId } = req.body;
        const anwserdb = await Answer.findOne({ where: { ChoiceId:choiceId, TheoryTrainAttemptId:theoryTrainAttemptsId } });

        if (anwserdb)
            return res.status(409).json({
                success: false,
                message: "Answer already exists",
            });


        const anwsers = await Answer.create({
            TheoryTrainAttemptId: theoryTrainAttemptsId,
            status,
            ChoiceId: choiceId

        });

        if (anwsers) {

            return res.status(201).json({
                success: true,
                message: "New Answer has been added",
                result: anwsers,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
