const Blog = require("../models/Blog");
const fs = require('fs');

const { Op } = require("sequelize");
const User = require("../models/User");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ title: { [Op.like]: whereSearch } }],
                }
                : {},include:{model:User}
        };
        const info = await Blog.paginate(options);
        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no Blog!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Blogs info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await Blog.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "Blog not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "Blog info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateBlog = async (req, res) => {
    try {
        const id = req.body.id;

        const title = req.body.title == undefined ? undefined : req.body.title;
        const content = req.body.content == undefined ? undefined : req.body.content;
        const userId = req.body.country == undefined ? undefined : req.body.country;
        const status = req.body.status == undefined ? undefined : req.body.status;
        const notes = req.body.email == undefined ? undefined : req.body.email;
        const image = req.files === undefined || req.files.length === 0 ? "" : req.files.map(files => files.filename).join(",")
        const blog = await Blog.findByPk(id);

        const previousUrls = (blog.image || '').split(',');


        const result = await Blog.update(
            {
                title,
                content,
                UserId: userId,
                status,
                notes,
                image
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {

            const dir1 = `./public/assets/images/blogs/${id}/`;


            // Remove old images if they exist
            previousUrls.forEach(previousUrl => {
                const filePath = dir1 + previousUrl.trim();
                if (fs.existsSync(filePath)) {
                    fs.rmSync(filePath);
                }
            });
            return res.status(200).json({
                success: true,
                message: `Blog ${req.body.title} is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for blog: ${req.body.title}, may be column not exists`,
            });
        }
    } catch (err) {

        const dir1 = `./public/assets/images/blogs/${req.query.id}/`;

        if (req.files !== undefined && req.files.length !== 0) {
            req.files.forEach(file => {
                if (fs.existsSync(dir1 + file.filename)) {
                    fs.rmSync(dir1 + file.filename, { recursive: true });
                }
            });
        }
        return res.status(500).json({
            success: false,
            message: err.errors
                ? err.errors.map((item) => item.message)
                : [err.message],
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteBlogById = async (req, res) => {
    try {
        const id = req.query.id;
        const blog = await Blog.findByPk(id);

        if (blog) {
            await blog.destroy();
            if (blog > 0) {
                const dir1 = `./public/assets/images/blogs/${req.query.id}`;
                if (fs.existsSync(dir1)) {
                    fs.rmSync(dir1, { recursive: true });
                }
                return res.status(200).json({
                    success: true,
                    message: `blog with id=${id} was deleted successfully!`,
                });
            }
        } else {
            return res.status(404).json({
                success: false,
                message: "blog does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addBlog = async (req, res) => {
    try {
        let dir1 = null
        const id = req.query.id
        const { title, content, userId, status, notes } = req.body;
        const files = req.files === undefined ? "" : req.files.map(file => file.filename).join(",")
        const blogdb = await Blog.findOne({ where: { title, content } });

        if (blogdb) {
            dir1 = `./public/assets/images/blogs/${req.query.id}/`;
            if (req.files !== undefined && req.files.length !== 0) {
                req.files.forEach(file => {
                    if (fs.existsSync(dir1 + file.filename)) {
                        fs.rmSync(dir1 + file.filename, { recursive: true });
                    }
                });
            }
            return res.status(409).json({
                success: false,
                message: "blog already exists",
            });
        }

        const blog = await Blog.create({
            id,
            title,
            content,
            UserId: userId,
            status,
            notes,
            image: files

        });

        if (blog) {

            return res.status(201).json({
                success: true,
                message: "New blog has been added",
                result: blog,
            });
        }
    } catch (err) {
        const dir1 = `./public/assets/images/blogs/${req.query.id}/`;

        if (req.files !== undefined && req.files.length !== 0) {
            req.files.forEach(file => {
                if (fs.existsSync(dir1 + file.filename)) {
                    fs.rmSync(dir1 + file.filename, { recursive: true });
                }
            });
        }
        return res.status(500).json({
            success: false,
            message: err.errors
                ? err.errors.map((item) => item.message)
                : [err.message],
        });
    }
};
