const Choice = require("../models/Choice");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ choice: { [Op.like]: whereSearch } }],
                }
                : {},
           };
        const info = await Choice.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no Choice!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Choices info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await Choice.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "Choice not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "Choice info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateChoice = async (req, res) => {
    try {
        const id = req.body.id;

        const questionId = req.body.questionId == undefined ? undefined : req.body.questionId;
        const choice = req.body.choice == undefined ? undefined : req.body.choice;
        const status = req.body.status == undefined ? undefined : req.body.status;
       



        const result = await Choice.update(
            {
               QuestionId:questionId,
               choice,
               status
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {
           

            return res.status(200).json({
                success: true,
                message: `Choice is updated`,
            });
        } else {
           
            return res.status(204).json({
                success: false,
                message: `nothing updated for Choice, may be column not exists`,
            });
        }
    } catch (err) {
        
        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteChoiceById = async (req, res) => {
    try {
        const id = req.query.id;
        const choice = await Choice.findByPk(id);

        if (choice) {
            await choice.destroy();
            
            return res.status(200).json({
                success: true,
                message: `Choice with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "Choice does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({ success: false, message: err.message });

    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addChoice = async (req, res) => {
    try {
        const { questionId,choice,status } = req.body;
        const choicedb = await Choice.findOne({ where: { choice,questionId } });

        if (choicedb)
        return res.status(409).json({
            success: false,
                message: "Choice already exists",
            });

       
        const choices = await Choice.create({
            QuestionId:questionId,
            status,
            choice,
            
        });

        if (choices) {
            
            return res.status(201).json({
                success: true,
                message: "New Choice has been added",
                result: choices,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
