const Answer = require("../models/Answer");
const Choice = require("../models/Choice");
const Question = require("../models/Question");
const Sub = require("../models/Sub");
const TheoryTrain = require("../models/TheoryTrain");
const TheoryTrainAttempts = require("../models/TheoryTrainAttempts");
const User = require("../models/User");
const UserSub = require("../models/UserSub");
const UserTheoryTrain = require("../models/UserTheoryTrain");

exports.attempt = async (req, res) => {
    try {
        const userId = req.query.userId
        const theoryTrainId = req.query.theoryTrainId
        const currentDate = new Date()

        const theoryTrain = await TheoryTrain.findOne({ where: { id: theoryTrainId } })

        const userSub = await UserSub.findOne({ where: { UserId: userId } })

        if (!userSub) {
            return res.status(200)
                .json({
                    success: false,
                    theoryTrain,
                    message: "you are not subscribed",
                    lastAttempt: null,
                    questionsWithAnswers: [],
                    questionsWithNoAnswers: []
                });

        } else {
            const sub = await Sub.findOne({ where: { id: userSub.SubId } });
            if (sub.type == 'Basic') {
                if (userSub.subExpiryDate <= currentDate)
                    return res.status(200)
                        .json({
                            success: false,
                            theoryTrain,
                            message: "Your subscription has been expired",
                            lastAttempt: null,
                            questionsWithAnswers: [],
                            questionsWithNoAnswers: []
                        });
                else {
                    let userTheoryTrain = await UserTheoryTrain.findOne({ where: { UserId: userId, TheoryTrainId: theoryTrainId } })

                    if (!userTheoryTrain) {
                        const tomorrow = new Date(currentDate)
                        tomorrow.setDate(tomorrow.getDate() + 1)
                        userTheoryTrain = await UserTheoryTrain.create({
                            TheoryTrainId: theoryTrainId,
                            UserId: userId,
                            status: false,
                            consumedAttempts: 0,
                            endAt: tomorrow

                        })
                    }
                    // if (theoryTrain.period == 0) {  
                    //     return res.status(200)
                    //     .json({
                    //         success: false,
                    //         theoryTrain,
                    //         message: "Your period has been 0",
                    //         lastAttempt: null,
                    //         questionsWithAnswers: [],
                    //         questionsWithNoAnswers: []
                    //     });
                    // }
                    if (userTheoryTrain.endAt > currentDate) {
                        if (userTheoryTrain.consumedAttempts < sub.attempts) {
                            let userAttempt = await TheoryTrainAttempts.findOne({
                                where: { UserTheoryTrainId: userTheoryTrain.id, score: null },
                                order: [['endAt', 'DESC']] // Sort by endAt column in descending order
                            });
                            if (!userAttempt) {
                                userTheoryTrain.consumedAttempts += 1
                                await userTheoryTrain.save()

                                const hoursToAdd = parseInt(theoryTrain.period);
                                let newEndAtDate = new Date(currentDate);
                                if (theoryTrain.period == 0) {
                                    newEndAtDate = new Date(userSub.subExpiryDate);
                                }
                                newEndAtDate.setHours(newEndAtDate.getHours() + hoursToAdd);

                                userAttempt = await TheoryTrainAttempts.create({
                                    UserTheoryTrainId: userTheoryTrain.id,
                                    endAt: newEndAtDate,
                                    score: null,
                                    status: false
                                })
                                const questions = await Question.findAll({
                                    where: { TheoryTrainId: theoryTrainId },
                                    include: { model: Choice }
                                });

                                return res.status(200).json({
                                    success: true,
                                    theoryTrain,
                                    isFullTime: theoryTrain.period == 0,

                                    message: "new attempt has been assigned",
                                    lastAttempt: userAttempt,
                                    questionsWithAnswers: questions,
                                    questionsWithNoAnswers: []

                                });
                            } else {
                                if (userAttempt.score !== null) {
                                    //new attemp
                                    userTheoryTrain.consumedAttempts += 1
                                    await userTheoryTrain.save()
                                    const hoursToAdd = parseInt(theoryTrain.period);
                                    let newEndAtDate = new Date(currentDate);
                                    if (theoryTrain.period == 0) {
                                        newEndAtDate = new Date(userSub.subExpiryDate);
                                    }
                                    newEndAtDate.setHours(newEndAtDate.getHours() + hoursToAdd);

                                    userAttempt = await TheoryTrainAttempts.create({
                                        UserTheoryTrainId: userTheoryTrain.id,
                                        endAt: newEndAtDate,
                                        score: null,
                                        status: false
                                    })
                                    const questions = await Question.findAll({
                                        where: { TheoryTrainId: theoryTrainId },
                                        include: { model: Choice }
                                    });

                                    return res.status(200).json({
                                        success: true,
                                        theoryTrain, isFullTime: theoryTrain.period == 0,
                                        message: "new attempt has been assigned",
                                        lastAttempt: userAttempt,
                                        questionsWithAnswers: questions,
                                        questionsWithNoAnswers: []
                                    });
                                }
                            }


                            if (userAttempt.endAt > currentDate) {


                                // Fetch all questions for the theory train
                                const questions = await Question.findAll({
                                    where: { TheoryTrainId: theoryTrainId },
                                    include: { model: Choice }
                                });

                                // Initialize arrays to store questions with answers and questions with no answers
                                let questionsWithAnswers = [];
                                let questionsWithNoAnswers = [];

                                // Iterate over each question to include the associated answer, if any
                                for (const question of questions) {
                                    // Find the answer associated with the current question in the user's attempt
                                    const answer = await Answer.findOne({
                                        where: { ChoiceId: question.Choices.map(choice => choice.id), TheoryTrainAttemptId: userAttempt.id }
                                    });

                                    if (answer) {
                                        // If there is an answer, include the question along with the selected answer id
                                        const selectedChoice = question.Choices.find(choice => choice.id === answer.ChoiceId);
                                        const questionWithAnswer = {
                                            id: question.id,
                                            question: question.question,
                                            score: question.score,
                                            img: question.img,
                                            status: question.status,
                                            TheoryTrainId: question.TheoryTrainId,
                                            Choices: question.Choices.map(choice => ({
                                                id: choice.id,
                                                choice: choice.choice,
                                                status: choice.status,
                                                QuestionId: choice.QuestionId
                                            })),
                                            answer: answer // Include the entire answer instance
                                        };
                                        questionsWithAnswers.push(questionWithAnswer);
                                    } else {
                                        // If there is no answer, include the question with choices only
                                        questionsWithNoAnswers.push(question);
                                    }
                                }

                                // Return both questions with answers and questions with no answers
                                return res.status(200).json({
                                    success: true,
                                    theoryTrain,
                                    isFullTime: theoryTrain.period == 0,
                                    message: "Last attempt details fetched successfully",
                                    lastAttempt: userAttempt,
                                    questionsWithAnswers: questionsWithAnswers,
                                    questionsWithNoAnswers: questionsWithNoAnswers
                                });



                            } else {
                                const lastAttempt = userAttempt
                                const answers = await Answer.findAll({
                                    where: { TheoryTrainAttemptId: lastAttempt.id },
                                    include: {
                                        model: Choice,
                                        where: { status: true } // Filter only correct choices
                                    }
                                });

                                // Calculate the total score based on the correct choices and their corresponding question scores
                                let totalScore = 0;
                                for (const answer of answers) {
                                    // Find the corresponding question for each answer
                                    const question = await Question.findByPk(answer.Choice.QuestionId);
                                    if (question) {
                                        // Add the score of the question to the total score
                                        totalScore += question.score;
                                    }
                                }

                                // Update the last attempt with the calculated total score
                                await lastAttempt.update({ score: totalScore });


                                userTheoryTrain.consumedAttempts += 1
                                await userTheoryTrain.save()
                                const hoursToAdd = parseInt(theoryTrain.period);
                                let newEndAtDate = new Date(currentDate);
                                if (theoryTrain.period == 0) {
                                    newEndAtDate = new Date(userSub.subExpiryDate);
                                }
                                newEndAtDate.setHours(newEndAtDate.getHours() + hoursToAdd);

                                const newUserAttempt = await TheoryTrainAttempts.create({
                                    UserTheoryTrainId: userTheoryTrain.id,
                                    endAt: newEndAtDate,
                                    score: null,
                                    status: false
                                })

                                const questions = await Question.findAll({
                                    where: { TheoryTrainId: theoryTrainId },
                                    include: { model: Choice }
                                });

                                return res.status(200).json({
                                    success: true,
                                    theoryTrain,
                                    isFullTime: theoryTrain.period == 0,
                                    message: "new attempt has been assigned,and last one calculated",
                                    lastAttempt: lastAttempt,
                                    questionsWithAnswers: questions,
                                    questionsWithNoAnswers: []
                                });
                            }


                        } else {
                            let userAttempt = await TheoryTrainAttempts.findOne({
                                where: { UserTheoryTrainId: userTheoryTrain.id, score: null },
                                order: [['endAt', 'DESC']] // Sort by endAt column in descending order
                            });
                            if (userAttempt == null) {
                                return res.status(200).json({
                                    success: false,
                                    theoryTrain,
                                    isFullTime:theoryTrain.period == 0,
                                    message: "you have used all your attempts for this day",
                                    lastAttempt: null,
                                    theoryTrainAttempts: await TheoryTrainAttempts.findAll(
                                        {
                                            where: {
                                                UserTheoryTrainId: userTheoryTrain.id,
                                                // score: null
                                            },
                                            order: [['endAt', 'DESC']] // Sort by endAt column in descending order
                                        }
                                    ),
                                    questionsWithAnswers: [],
                                    questionsWithNoAnswers: []
                                });
                            }

                            if (userAttempt.endAt < currentDate) {

                                const lastAttempt = userAttempt

                                if (lastAttempt.score == null) {
                                    const answers = await Answer.findAll({
                                        where: { TheoryTrainAttemptId: lastAttempt.id },
                                        include: {
                                            model: Choice,
                                            where: { status: true } // Filter only correct choices
                                        }
                                    });

                                    // Calculate the total score based on the correct choices and their corresponding question scores
                                    let totalScore = 0;
                                    for (const answer of answers) {
                                        // Find the corresponding question for each answer
                                        const question = await Question.findByPk(answer.Choice.QuestionId);
                                        if (question) {
                                            // Add the score of the question to the total score
                                            totalScore += question.score;
                                        }
                                    }

                                    // Update the last attempt with the calculated total score
                                    await lastAttempt.update({ score: totalScore });
                                    return res.status(200)
                                        .json({
                                            success: false,
                                            theoryTrain,
                                            isFullTime:theoryTrain.period == 0,
                                            message: "you have used all your attempts for this day,and last one calculated",
                                            lastAttempt: null,
                                            theoryTrainAttempts: await TheoryTrainAttempts.findAll(
                                                {
                                                    where: {
                                                        UserTheoryTrainId: userTheoryTrain.id,
                                                        // score: null
                                                    },
                                                    order: [['endAt', 'DESC']] // Sort by endAt column in descending order
                                                }
                                            ),
                                            questionsWithAnswers: [],
                                            questionsWithNoAnswers: []
                                        });

                                } else {
                                    return res.status(200)
                                        .json({
                                            success: false,
                                            theoryTrain,
                                            isFullTime:theoryTrain.period == 0,
                                            message: "you have used all your attempts for this day",
                                            lastAttempt: null,
                                            theoryTrainAttempts: await TheoryTrainAttempts.findAll(
                                                {
                                                    where: {
                                                        UserTheoryTrainId: userTheoryTrain.id,
                                                        // score: null
                                                    },
                                                    order: [['endAt', 'DESC']] // Sort by endAt column in descending order
                                                }
                                            ),
                                            questionsWithAnswers: [],
                                            questionsWithNoAnswers: []
                                        });
                                }
                            } else {
                                // Fetch all questions for the theory train
                                const questions = await Question.findAll({
                                    where: { TheoryTrainId: theoryTrainId },
                                    include: { model: Choice }
                                });

                                // Initialize arrays to store questions with answers and questions with no answers
                                let questionsWithAnswers = [];
                                let questionsWithNoAnswers = [];

                                // Iterate over each question to include the associated answer, if any
                                for (const question of questions) {
                                    // Find the answer associated with the current question in the user's attempt
                                    const answer = await Answer.findOne({
                                        where: { ChoiceId: question.Choices.map(choice => choice.id), TheoryTrainAttemptId: userAttempt.id }
                                    });

                                    if (answer) {
                                        // If there is an answer, include the question along with the selected answer id
                                        const selectedChoice = question.Choices.find(choice => choice.id === answer.ChoiceId);
                                        const questionWithAnswer = {
                                            id: question.id,
                                            question: question.question,
                                            score: question.score,
                                            img: question.img,
                                            status: question.status,
                                            TheoryTrainId: question.TheoryTrainId,
                                            Choices: question.Choices.map(choice => ({
                                                id: choice.id,
                                                choice: choice.choice,
                                                status: choice.status,
                                                QuestionId: choice.QuestionId
                                            })),
                                            answer: answer // Include the entire answer instance
                                        };
                                        questionsWithAnswers.push(questionWithAnswer);
                                    } else {
                                        // If there is no answer, include the question with choices only
                                        questionsWithNoAnswers.push(question);
                                    }
                                }

                                // Return both questions with answers and questions with no answers
                                return res.status(200).json({
                                    success: true,
                                    theoryTrain,
                                    isFullTime:theoryTrain.period == 0,
                                    message: "Last attempt details fetched successfully",
                                    lastAttempt: userAttempt,
                                    questionsWithAnswers: questionsWithAnswers,
                                    questionsWithNoAnswers: questionsWithNoAnswers
                                });
                            }

                        }
                    } else {

                        // create a new usertheorytrain for the new day
                        const tomorrow = new Date(currentDate)
                        tomorrow.setDate(tomorrow.getDate() + 1)
                        userTheoryTrain = await UserTheoryTrain.create({
                            TheoryTrainId: theoryTrainId,
                            UserId: userId,
                            status: false,
                            consumedAttempts: 0,
                            endAt: tomorrow

                        })

                        userTheoryTrain.consumedAttempts += 1
                        await userTheoryTrain.save()
                        const hoursToAdd = parseInt(theoryTrain.period);
                        let newEndAtDate = new Date(currentDate);
                        if (theoryTrain.period == 0) {
                            newEndAtDate = new Date(userSub.subExpiryDate);
                        }
                        newEndAtDate.setHours(newEndAtDate.getHours() + hoursToAdd);

                        const userAttempt = await TheoryTrainAttempts.create({
                            UserTheoryTrainId: userTheoryTrain.id,
                            endAt: newEndAtDate,
                            score: null,
                            status: false
                        })
                        const questions = await Question.findAll({
                            where: { TheoryTrainId: theoryTrainId },
                            include: { model: Choice }
                        });

                        return res.status(200).json({
                            success: true,
                            theoryTrain,
                            isFullTime:theoryTrain.period == 0,
                            message: "new attempt has been assigned",
                            details: questions,
                            lastAttempt: userAttempt
                        });


                    }
                }
            } else if (sub.type == 'Extra') {

                if (userSub.subExpiryDate <= currentDate)
                    return res.status(200)
                        .json({
                            success: false,
                            theoryTrain,
                            isFullTime:theoryTrain.period == 0,
                            message: "Your subscription has been expired",
                            lastAttempt: null,
                            questionsWithAnswers: [],
                            questionsWithNoAnswers: []
                        });
                else {
                    let userTheoryTrain = await UserTheoryTrain.findOne({ where: { UserId: userId, TheoryTrainId: theoryTrainId } })
                    if (!userTheoryTrain) {

                        userTheoryTrain = await UserTheoryTrain.create({
                            TheoryTrainId: theoryTrainId,
                            UserId: userId,
                            status: false,
                            consumedAttempts: 0,
                            endAt: 0

                        })
                    }

                    let userAttempt = await TheoryTrainAttempts.findOne({
                        where: { UserTheoryTrainId: userTheoryTrain.id },
                        order: [['endAt', 'DESC']] // Sort by endAt column in descending order
                    });
                    if (!userAttempt) {
                        const hoursToAdd = parseInt(theoryTrain.period);
                        let newEndAtDate = new Date(currentDate);
                        if (theoryTrain.period == 0) {
                            newEndAtDate = new Date(userSub.subExpiryDate);
                        }
                        newEndAtDate.setHours(newEndAtDate.getHours() + hoursToAdd);

                        userAttempt = await TheoryTrainAttempts.create({
                            UserTheoryTrainId: userTheoryTrain.id,
                            endAt: newEndAtDate,
                            score: null,
                            status: false
                        })
                        const questions = await Question.findAll({
                            where: { TheoryTrainId: theoryTrainId },
                            include: { model: Choice }
                        });

                        return res.status(200).json({
                            success: true,
                            theoryTrain,
                            isFullTime:theoryTrain.period == 0,
                            message: "new attempt has been assigned",
                            lastAttempt: userAttempt,
                            questionsWithAnswers: questions,
                            questionsWithNoAnswers: []
                        });
                    } else {
                        if (userAttempt.score !== null) {
                            //new attemp
                            const hoursToAdd = parseInt(theoryTrain.period);
                            let newEndAtDate = new Date(currentDate);
                            if (theoryTrain.period == 0) {
                                newEndAtDate = new Date(userSub.subExpiryDate);
                            }
                            newEndAtDate.setHours(newEndAtDate.getHours() + hoursToAdd);

                            userAttempt = await TheoryTrainAttempts.create({
                                UserTheoryTrainId: userTheoryTrain.id,
                                endAt: newEndAtDate,
                                score: null,
                                status: false
                            })
                            const questions = await Question.findAll({
                                where: { TheoryTrainId: theoryTrainId },
                                include: { model: Choice }
                            });

                            return res.status(200).json({
                                success: true,
                                theoryTrain,
                                isFullTime:theoryTrain.period == 0,
                                message: "Last attempt details fetched successfully",
                                lastAttempt: userAttempt,
                                questionsWithAnswers: questions,
                                questionsWithNoAnswers: []
                            });

                        }
                        //return last attempt

                        // Fetch all questions for the theory train
                        const questions = await Question.findAll({
                            where: { TheoryTrainId: theoryTrainId },
                            include: { model: Choice }
                        });

                        // Initialize arrays to store questions with answers and questions with no answers
                        let questionsWithAnswers = [];
                        let questionsWithNoAnswers = [];

                        // Iterate over each question to include the associated answer, if any
                        for (const question of questions) {
                            // Find the answer associated with the current question in the user's attempt
                            const answer = await Answer.findOne({
                                where: { ChoiceId: question.Choices.map(choice => choice.id), TheoryTrainAttemptId: userAttempt.id }
                            });

                            if (answer) {
                                // If there is an answer, include the question along with the selected answer id
                                const selectedChoice = question.Choices.find(choice => choice.id === answer.ChoiceId);
                                const questionWithAnswer = {
                                    id: question.id,
                                    question: question.question,
                                    score: question.score,
                                    img: question.img,
                                    status: question.status,
                                    TheoryTrainId: question.TheoryTrainId,
                                    Choices: question.Choices.map(choice => ({
                                        id: choice.id,
                                        choice: choice.choice,
                                        // status: choice.status,
                                        QuestionId: choice.QuestionId
                                    })),
                                    answer: answer // Include the entire answer instance
                                };
                                questionsWithAnswers.push(questionWithAnswer);
                            } else {
                                // If there is no answer, include the question with choices only
                                questionsWithNoAnswers.push(question);
                            }
                        }

                        // Return both questions with answers and questions with no answers
                        return res.status(200).json({
                            success: true,
                            theoryTrain,
                            isFullTime:theoryTrain.period == 0,
                            message: "Last attempt details fetched successfully",
                            lastAttempt: userAttempt,
                            questionsWithAnswers: questionsWithAnswers,
                            questionsWithNoAnswers: questionsWithNoAnswers
                        });



                    }
                }
            }
        }
    }
    catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
}


exports.submit = async (req, res) => {
    // try {
    const theoryTrainAttemptId = req.query.theoryTrainAttemptId; // Correcting the retrieval of userTheoryTrainId from the query

    // Find the last attempt for the user's theory train
    const lastAttempt = await TheoryTrainAttempts.findOne({
        where: { id: theoryTrainAttemptId }
    }); 
    // Check if the exam is already finished
    if (lastAttempt && lastAttempt.score !== null) {
        return res.status(200).json({
            success: true,
            message: "This exam is already finished",
        });
    }

    // Fetch all answers associated with the last attempt
    const answers = await Answer.findAll({
        where: { TheoryTrainAttemptId: lastAttempt.id },
        include: {
            model: Choice,
            where: { status: true } // Filter only correct choices
        }
    });

    // Calculate the total score based on the correct choices and their corresponding question scores
    let totalScore = 0;
    for (const answer of answers) {
        // Find the corresponding question for each answer
        const question = await Question.findByPk(answer.Choice.QuestionId);
        if (question) {
            // Add the score of the question to the total score
            totalScore += question.score;
        }
    }

    // Update the last attempt with the calculated total score
    await lastAttempt.update({ score: totalScore });

    return res.status(200).json({
        success: true,
        message: "Exam score calculated and saved successfully",
    });
    // } catch (err) {
    //     return res.status(500).json({
    //         success: false,
    //         message: err.message,
    //     });
    // }
};