const Faq = require("../models/Faq");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ title: { [Op.like]: whereSearch } }],
                }
                : {},
           };
        const info = await Faq.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no Faq!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Faqs info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await Faq.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "Faq not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "Faq info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateFaq = async (req, res) => {
    try {
        const id = req.body.id;

        const title = req.body.title == undefined ? undefined : req.body.title;
        const description = req.body.description == undefined ? undefined : req.body.description;
        const status = req.body.status == undefined ? undefined : req.body.status;
       



        const result = await Faq.update(
            {
               title,
               status,
               description
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {
           

            return res.status(200).json({
                success: true,
                message: `Faq ${req.body.title} is updated`,
            });
        } else {
           
            return res.status(204).json({
                success: false,
                message: `nothing updated for Faq: ${req.body.title}, may be column not exists`,
            });
        }
    } catch (err) {
        
        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteFaqById = async (req, res) => {
    try {
        const id = req.query.id;
        const faq = await Faq.findByPk(id);

        if (faq) {
            await faq.destroy();
            
            return res.status(200).json({
                success: true,
                message: `Faq with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "Faq does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({ success: false, message: err.message });

    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addFaq = async (req, res) => {
    try {
        const { title,description,status } = req.body;
        const Faqdb = await Faq.findOne({ where: { title,description } });

        if (Faqdb)
        return res.status(409).json({
            success: false,
                message: "Faq already exists",
            });

       
        const faq = await Faq.create({
            title,
            description,
            status,
            
        });

        if (faq) {
            
            return res.status(201).json({
                success: true,
                message: "New Faq has been added",
                result: faq,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
