const PracList = require("../models/PracTrainPlayList");

const { Op } = require("sequelize");
const PracVideo = require("../models/PracTrainVideo");
const PreviewVideo = require("../models/PreviewVideo");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ title: { [Op.like]: whereSearch } }],
                }
                : {},
        };
        const info = await PracList.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no PracList!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All PracLists info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await PracList.findOne({
            where: { id: id },
            include: { model: PracVideo, include: { model: PreviewVideo } }
        });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "PracList not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "PracList info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updatePracList = async (req, res) => {
    try {
        const id = req.body.id;

        const title = req.body.title == undefined ? undefined : req.body.title;
        const description = req.body.description == undefined ? undefined : req.body.description;
        const playListUrl = req.body.playListUrl == undefined ? undefined : req.body.playListUrl;
        const status = req.body.status == undefined ? undefined : req.body.status;




        const result = await PracList.update(
            {
                title,
                status,
                description,
                playListUrl,
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `PracList is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for PracList, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deletePracListById = async (req, res) => {
    try {
        const id = req.query.id;
        const pracList = await PracList.findByPk(id);

        if (pracList) {
            await pracList.destroy();

            return res.status(200).json({
                success: true,
                message: `PracList with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "PracList does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addPracList = async (req, res) => {
    try {
        const { title, description, status, playListUrl } = req.body;
        const pracListdb = await PracList.findOne({ where: { title, description } });

        if (pracListdb)
            return res.status(409).json({
                success: false,
                message: "PracList already exists",
            });


        const pracList = await PracList.create({
            title,
            description,
            status,
            playListUrl

        });

        if (pracList) {

            return res.status(201).json({
                success: true,
                message: "New PracList has been added",
                result: pracList,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
