const PracVideo = require("../models/PracTrainVideo");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ title: { [Op.like]: whereSearch } }],
                }
                : {},
        };
        const info = await PracVideo.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no PracVideo!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All PracVideo info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await PracVideo.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "PracVideo not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "PracVideo info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updatePracvideo = async (req, res) => {
    try {
        const id = req.body.id;

        const title = req.body.title == undefined ? undefined : req.body.title;
        const description = req.body.description == undefined ? undefined : req.body.description;
        const videoUrl = req.body.videoUrl == undefined ? undefined : req.body.videoUrl;
        const status = req.body.status == undefined ? undefined : req.body.status;
        const pracListId = req.body.pracListId == undefined ? undefined : req.body.pracListId;




        const result = await PracVideo.update(
            {
                title,
                status,
                description,
                videoUrl,
                PracListId: pracListId
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `PracVideo is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for PracVideo, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deletePracVideotById = async (req, res) => {
    try {
        const id = req.query.id;
        const pracVideo = await PracVideo.findByPk(id);

        if (pracVideo) {
            await pracVideo.destroy();

            return res.status(200).json({
                success: true,
                message: `PracVideo with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "PracVideo does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addPracVideo = async (req, res) => {
    try {
        const { title, description, status, videoUrl,pracListId } = req.body;
        const pracVideodb = await PracVideo.findOne({ where: { title, description } });

        if (pracVideodb)
        return res.status(409).json({
            success: false,
                message: "PracVideo already exists",
            });


        const pracVideo = await PracVideo.create({
            title,
            description,
            status,
            videoUrl,
            PracListId:pracListId

        });

        if (PracVideo) {

            return res.status(201).json({
                success: true,
                message: "New PracVideo has been added",
                result: pracVideo,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
