const PreviewVideo = require("../models/PreviewVideo");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    // [Op.or]: [{ type: { [Op.like]: whereSearch } }],
                }
                : {},
        };
        const info = await PreviewVideo.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no PreviewVideo!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All PreviewVideo info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await PreviewVideo.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "PreviewVideo not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "PreviewVideo info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updatePrevVideo = async (req, res) => {
    try {
        const id = req.body.id;

        const pracVideoId = req.body.pracVideoId == undefined ? undefined : req.body.pracVideoId;
        const userId = req.body.userId == undefined ? undefined : req.body.userId;
        const status = req.body.status == undefined ? undefined : req.body.status;




        const result = await PreviewVideo.update(
            {
                UserId:userId,
                PracticeVideoId: pracVideoId,
                status

            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `PreviewVideo ${req.body.type} is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for PreviewVideo: ${req.body.type}, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deletePrevVideoById = async (req, res) => {
    try {
        const id = req.query.id;
        const previewVideo = await PreviewVideo.findByPk(id);

        if (previewVideo) {
            await previewVideo.destroy();

            return res.status(200).json({
                success: true,
                message: `PreviewVideo with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "PreviewVideo does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addPrevVideo = async (req, res) => {
    try {
        const { userId,pracVideoId, status } = req.body;
        const prevdb = await PreviewVideo.findOne({ where: { UserId:userId, PracVideoId:pracVideoId } });

        if (prevdb)
        return res.status(409).json({
            success: false,
                message: "PreviewVideo already exists",
            });


        const previewVideo = await PreviewVideo.create({
            UserId:userId,
            PracVideoId: pracVideoId,
            status

        });

        if (previewVideo) {

            return res.status(201).json({
                success: true,
                message: "New PreviewVideo has been added",
                result: previewVideo,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
