const Question = require("../models/Question");
const fs = require('fs');
const sequelize = require('../../config/database')

const { Op, where } = require("sequelize");
const Choice = require("../models/Choice");
const Answer = require("../models/Answer");
const UserTheoryTrain = require("../models/UserTheoryTrain");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const theorytrainId = req.query.theorytrainId;
        const searchQuery = req.query.search;
        const whereSearch = searchQuery ? `%${searchQuery}%` : "%%";

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: {},
            include: { model: Choice, include: { model: Answer } }
        };

        if (searchQuery) {
            options.where[Op.or] = [{ question: { [Op.like]: whereSearch } }];
        }

        if (theorytrainId) {
            options.where.TheoryTrainId = theorytrainId;
        }

        const info = await Question.paginate(options);

        if (!info || info.docs.length === 0) {
            return res.status(404).json({
                success: false,
                message: "No questions found for the provided criteria.",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Questions info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};


// //!  =-=-=-=-=-=-=-=-=-=  GET ALL By theoryTrain =-=-=-=-=-=-=-=-=-=-=//

// exports.showAllBytheoryTrain = async (req, res) => {
//     try {
//         const theorytrainId = req.query.theorytrainId;
//         const whereSearch =
//             req.query.search == null ? "%%" : `%${req.query.search}%`;

//         const options = {
//             page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
//             paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
//             where: req.query.search
//                 ? {
//                     [Op.or]: [],
//                 }
//                 : {},include:{model:Choice,include:{model:Answer}}
//         };
//         const info = await Question.paginate(options);

//         if (info == null) {
//             return res.status(204).json({
//                 success: false,
//                 message: "There is no Question!",
//             });
//         } else {
//             return res.status(200).json({
//                 success: true,
//                 message: "All Questions info",
//                 details: info,
//             });
//         }
//     } catch (err) {
//         return res.status(500).json({
//             success: false,
//             message: err.message,
//         });
//     }
// };

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await Question.findOne({ where: { id: id }, include: { model: Choice } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "Question not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "Question info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateQuestion = async (req, res) => {
    try {
        const id = req.query.id;

        const question = req.body.question == undefined ? undefined : req.body.question;
        const score = req.body.score == undefined ? undefined : req.body.score;
        const img = req.files === undefined || req.files.length === 0 ? "" : req.files.map(files => files.filename).join(",")
        const status = req.body.status == undefined ? undefined : req.body.status;
        const theoryTrainId = req.body.theoryTrainId == undefined ? undefined : req.body.theoryTrainId;

        const questions = await Question.findByPk(id);

        const previousUrls = (questions.img || '').split(',');

        const result = await Question.update(
            {
                question,
                score,
                img,
                status,
                TheoryTrainid: theoryTrainId,
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {
            const dir1 = `./public/assets/images/questions/${id}/`;


            // Remove old images if they exist
            previousUrls.forEach(previousUrl => {
                const filePath = dir1 + previousUrl.trim();
                if (fs.existsSync(filePath)) {
                    fs.rmSync(filePath);
                }
            });

            return res.status(200).json({
                success: true,
                message: `Question ${questions.question} is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for Question: ${req.body.name}, may be column not exists`,
            });
        }
    } catch (err) {

        const dir1 = `./public/assets/images/questions/${req.query.id}/`;

        if (req.files !== undefined && req.files.length !== 0) {
            req.files.forEach(file => {
                if (fs.existsSync(dir1 + file.filename)) {
                    fs.rmSync(dir1 + file.filename, { recursive: true });
                }
            });
        }
        return res.status(500).json({
            success: false,
            message: err.errors
                ? err.errors.map((item) => item.message)
                : [err.message],
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteQuestionById = async (req, res) => {
    try {
        const id = req.query.id;
        const question = await Question.findByPk(id);

        if (question) {
            await question.destroy();
            if (question > 0) {
                const dir1 = `./public/assets/images/questions/${req.query.id}`;
                if (fs.existsSync(dir1)) {
                    fs.rmSync(dir1, { recursive: true });
                }
                return res.status(200).json({
                    success: true,
                    message: `Question with id=${id} was deleted successfully!`,
                });
            }
        } else {
            return res.status(404).json({
                success: false,
                message: "Question does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({

            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addQuestion = async (req, res) => {
    try {

        const q = await sequelize.transaction(async (t) => {

            const id = req.query.id;


            const { theoryTrainId, question, score, status, choices } = req.body;
            const parsedChoices = JSON.parse(choices)
            console.log("🚀 ~ exports.addQuestion= ~ req.body:", req.body)
            const files = req.files === undefined ? "" : req.files.map(file => file.filename).join(",");

            // Check if the question already exists
            const questionExists = await Question.findOne({ where: { question, score } });
            if (questionExists) {
                // Delete uploaded files if they exist
                const dir1 = `./public/assets/images/question/${req.query.id}/`;
                if (req.files !== undefined && req.files.length !== 0) {
                    req.files.forEach(file => {
                        if (fs.existsSync(dir1 + file.filename)) {
                            fs.rmSync(dir1 + file.filename, { recursive: true });
                        }
                    });
                }
                return res.status(409).json({
                    success: false,
                    message: "Question already exists",
                });
            }

            // Create the question
            const questionData = {
                id,
                question,
                TheoryTrainId: theoryTrainId,
                score,
                img: files,
                status
            };
            const newQuestion = await Question.create(questionData, { transaction: t });

            // Create choices if provided
            if (parsedChoices && parsedChoices.length > 0) {
                for (const choiceData of parsedChoices) {
                    await Choice.create({
                        choice: choiceData.choice,
                        status: choiceData.status,
                        QuestionId: newQuestion.id
                    }, { transaction: t });
                }
            }
            return { newQuestion: newQuestion }
        })
        if (q.newQuestion)
            return res.status(201).json({
                success: true,
                message: "New question and choices have been added",
            });

    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};



//!  =-=-=-=-=-=-=-=-=-=  GET ALL ByTrainId  =-=-=-=-=-=-=-=-=-=-=//

exports.showByTrainId = async (req, res) => {
    try {
        const Id = req.query.theoryTrainId
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;
        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ question: { [Op.like]: whereSearch } }],
                }
                : {},
        };
        const info = await Question.paginate(options, { where: { TheoryTrainId: Id } });

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no Question!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Questions info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//
exports.showQStatus = async (req, res) => {
    try {
        const userTheoryTrainId = req.query.userTheoryTrainId;
        const whereSearch = req.query.search == null ? "%%" : `%${req.query.search}%`;

        // Find the UserTheoryTrain instance with associated answers, choices, and questions
        const userTheoryTrain = await UserTheoryTrain.findOne({
            where: {
                id: userTheoryTrainId // Filter by userTheoryTrainId
            },
            include: [
                {
                    model: Answer,
                    include: [
                        {
                            model: Choice,
                            include: [
                                {
                                    model: Question,
                                    where: {
                                        question: {
                                            [Op.like]: whereSearch // Apply search filter to question text
                                        }
                                    },
                                    include: [Choice] // Include all choices for the question
                                }
                            ]
                        }
                    ]
                }
            ]
        });

        if (!userTheoryTrain) {
            return res.status(204).json({
                success: false,
                message: "No UserTheoryTrain found for the specified userTheoryTrainId!"
            });
        } else {
            const questionsWithAnswers = [];
            const questionsWithoutAnswers = [];

            // Iterate over the answers associated with the UserTheoryTrain
            userTheoryTrain.Answers.forEach(answer => {
                // Get the choice associated with the answer
                const choice = answer.Choice;
                if (choice) {
                    // Get the question associated with the choice
                    const question = choice.Question;
                    if (question) {
                        // Check if the question is already in questionsWithAnswers
                        const existingQuestionIndex = questionsWithAnswers.findIndex(q => q.question.id === question.id);
                        if (existingQuestionIndex === -1) {
                            // If the question is not already in questionsWithAnswers, add it with its choices
                            questionsWithAnswers.push({
                                question: question,
                                answer: answer
                            });
                        } else {
                            // If the question is already in questionsWithAnswers, just add the answer to its existing entry
                            questionsWithAnswers[existingQuestionIndex].answer = answer;
                        }
                    }
                }
            });

            // Find all questions associated with the UserTheoryTrain
            const allQuestions = await Question.findAll({
                where: {
                    TheoryTrainId: userTheoryTrain.TheoryTrainId // Filter by TheoryTrainId of the UserTheoryTrain
                }
            });

            // Iterate over all questions to find those without answers
            for (const question of allQuestions) {
                // Check if the question is not already included in questionsWithAnswers
                if (!questionsWithAnswers.some(q => q.question.id === question.id)) {
                    // Fetch all choices for the question
                    const choices = await Choice.findAll({
                        where: {
                            QuestionId: question.id
                        }
                    });
                    // Push the question along with its choices to questionsWithoutAnswers
                    questionsWithoutAnswers.push({
                        question: question,
                        choices: choices
                    });
                }
            }

            return res.status(200).json({
                success: true,
                message: "Questions info",
                details: {
                    withAnswers: questionsWithAnswers,
                    withoutAnswers: questionsWithoutAnswers
                }
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message
        });
    }
};







