const School = require("../models/School");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ name: { [Op.like]: whereSearch } }],
                }
                : {},
           };
        const info = await School.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no School!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Schools info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await School.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "School not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "School info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateSchool = async (req, res) => {
    try {
        const id = req.body.id;

        const name = req.body.name == undefined ? undefined : req.body.name;
        const address = req.body.address == undefined ? undefined : req.body.address;
        const phone = req.body.phone == undefined ? undefined : req.body.phone;
        const email = req.body.email == undefined ? undefined : req.body.email;
        const notes = req.body.notes == undefined ? undefined : req.body.notes;
        const status = req.body.status == undefined ? undefined : req.body.status;




        const result = await School.update(
            {
              name,
              address,
              phone,
              email,
              notes,
              status
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {
           

            return res.status(200).json({
                success: true,
                message: `School ${req.body.name} is updated`,
            });
        } else {
           
            return res.status(204).json({
                success: false,
                message: `nothing updated for School: ${req.body.name}, may be column not exists`,
            });
        }
    } catch (err) {
        
        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteSchoolById = async (req, res) => {
    try {
        const id = req.query.id;
        const school = await School.findByPk(id);

        if (school) {
            await school.destroy();
            
            return res.status(200).json({
                success: true,
                message: `School with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "School does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addSchool = async (req, res) => {
    try {
        const { name,address,phone,email,notes,status } = req.body;
        const Schooldb = await School.findOne({ where: { name,address } });

        if (Schooldb)
        return res.status(409).json({
            success: false,
                message: "School already exists",
            });

       
        const school = await School.create({
            name,
            address,
            phone,
            email,
            notes,
            status
            
        });

        if (school) {
            
            return res.status(201).json({
                success: true,
                message: "New School has been added",
                result: school,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
