const Sub = require("../models/Sub");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ type: { [Op.like]: whereSearch } }],
                }
                : {},


        };
        const info = await Sub.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no Sub!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All Subs info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await Sub.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "Sub not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "Sub info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateSub = async (req, res) => {
    try {
        const id = req.body.id;

        const type = req.body.type == undefined ? undefined : req.body.type;
        const description = req.body.description == undefined ? undefined : req.body.description;
        const attempts = req.body.attempts == undefined ? undefined : req.body.attempts;
        const price = req.body.price == undefined ? undefined : req.body.price;
        const status = req.body.status == undefined ? undefined : req.body.status;
        const features = req.body.features == undefined ? undefined : req.body.features;




        const result = await Sub.update(
            {
                type,
                description,
                attempts,
                price,
                status,
                features

            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `Sub ${req.body.type} is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for Sub: ${req.body.type}, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteSubById = async (req, res) => {
    try {
        const id = req.query.id;
        const sub = await Sub.findByPk(id);

        if (sub) {
            await sub.destroy();

            return res.status(200).json({
                success: true,
                message: `Sub with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "Sub does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addSub = async (req, res) => {
    try {
        const { type, description, attempts, price, status, features } = req.body;
        const subdb = await Sub.findOne({ where: { type, price } });

        if (subdb)
            return res.status(409).json({
                success: false,
                message: "Sub already exists",
            });


        const sub = await Sub.create({
            type,
            description,
            attempts,
            price,
            status,
            features

        });

        if (Sub) {

            return res.status(201).json({
                success: true,
                message: "New Sub has been added",
                result: sub,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
