const TheoryTrain = require("../models/TheoryTrain");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [{ title: { [Op.like]: whereSearch } }],
                }
                : {},
           };
        const info = await TheoryTrain.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no TheoryTrain!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All TheoryTrains info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await TheoryTrain.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "TheoryTrain not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "TheoryTrain info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateTheoryTrain = async (req, res) => {
    try {
        const id = req.body.id;

        const title = req.body.title == undefined ? undefined : req.body.title;
        const description = req.body.description == undefined ? undefined : req.body.description;
        const status = req.body.status == undefined ? undefined : req.body.status;
        const period = req.body.period == undefined ? undefined : req.body.period;
       



        const result = await TheoryTrain.update(
            {
               title,
               status,
               description,
               period
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {
           

            return res.status(200).json({
                success: true,
                message: `TheoryTrain ${req.body.title} is updated`,
            });
        } else {
           
            return res.status(204).json({
                success: false,
                message: `nothing updated for TheoryTrain: ${req.body.title}, may be column not exists`,
            });
        }
    } catch (err) {
        
        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteTheoryTrainById = async (req, res) => {
    try {
        const id = req.query.id;
        const thoryTrain = await TheoryTrain.findByPk(id);

        if (thoryTrain) {
            await thoryTrain.destroy();
            
            return res.status(200).json({
                success: true,
                message: `TheoryTrain with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "TheoryTrain does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addTheoryTrain = async (req, res) => {
    try {
        const { title,description,status,period } = req.body;
        const TheoryTraindb = await TheoryTrain.findOne({ where: { title,description } });

        if (TheoryTraindb)
        return res.status(409).json({
            success: false,
                message: "TheoryTrain already exists",
            });

       
        const thoryTrain = await TheoryTrain.create({
            title,
            description,
            status,
            period
            
        });

        if (thoryTrain) {
            
            return res.status(201).json({
                success: true,
                message: "New TheoryTrain has been added",
                result: thoryTrain,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
