const TheoryTrainAttempts = require("../models/TheoryTrainAttempts");

const { Op } = require("sequelize");
const UserTheoryTrain = require("../models/UserTheoryTrain");
const Answer = require("../models/Answer");
const TheoryTrain = require("../models/TheoryTrain");
const Question = require("../models/Question");
const Choice = require("../models/Choice");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    // [Op.or]: [{ title: { [Op.like]: whereSearch } }],
                }
                : {},
        };
        const info = await TheoryTrainAttempts.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no TheoryTrainAttempts!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All TheoryTrainAttemptss info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
exports.showAllByUserTheoryTrainAndUserId = async (req, res) => {
    try {

        const userId = req.query.userId
        const theoryTrainId = req.query.userTheoryTrainId

        const info = await TheoryTrainAttempts.findAll(
            {
                include:
                    [
                        {
                            model: UserTheoryTrain,
                            where: {
                                UserId: userId,
                                TheoryTrainId: theoryTrainId
                            },
                            include: {
                                model: TheoryTrain,
                                include: {
                                    model: Question,
                                    include: { all: true }
                                }
                            }
                        },
                        {
                            model: Answer,
                            include: { all: true }
                        },

                    ],
                order: [['endAt', 'DESC']] // Sort by endAt column in descending order
            }
        );
        return res.status(200).json({
            success: true,
            message: "All TheoryTrainAttemptss by userId and userTheoryTrain   info",
            details: info,
        });

    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await TheoryTrainAttempts.findOne({
            where: { id: id },
            include:
                [
                    {
                        model: UserTheoryTrain,
                        include: {
                            model: TheoryTrain,
                            include: {
                                model: Question,
                                include: {
                                    model: Choice,
                                },

                            }
                        }
                    },
                    {
                        model: Answer,
                        include: {
                            model: Choice,
                        },
                    },

                ],
            order: [['endAt', 'DESC']] // Sort by endAt column in descending order

        });


        return res.status(200).json({
            success: true,
            message: "TheoryTrainAttempts info",
            info: info,
        });

    } catch (err) {
        return res.status(404).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateTheoryTrainAttempts = async (req, res) => {
    try {
        const id = req.body.id;

        const score = req.body.score == undefined ? undefined : req.body.score;
        const endAt = req.body.endAt == undefined ? undefined : req.body.endAt;
        const status = req.body.status == undefined ? undefined : req.body.status;
        const userTheoryTrainId = req.body.userTheoryTrainId == undefined ? undefined : req.body.userTheoryTrainId;





        const result = await TheoryTrainAttempts.update(
            {
                score,
                UserTheoryTrainId: userTheoryTrainId,
                endAt,
                status

            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `TheoryTrainAttempts is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for TheoryTrainAttempts, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteTheoryTrainAttemptsById = async (req, res) => {
    try {
        const id = req.query.id;
        const theoryTrainAttempt = await TheoryTrainAttempts.findByPk(id);

        if (theoryTrainAttempt) {
            await theoryTrainAttempt.destroy();

            return res.status(200).json({
                success: true,
                message: `TheoryTrainAttempts with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "TheoryTrainAttempts does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addTheoryTrainAttempts = async (req, res) => {
    try {
        const { userTheoryTrainId, endAt, status, score } = req.body;
        const TheoryTrainAttemptsdb = await TheoryTrainAttempts.findOne({ where: { UserTheoryTrainId: userTheoryTrainId, endAt, score } });

        if (TheoryTrainAttemptsdb)
            return res.status(409).json({
                success: false,
                message: "TheoryTrainAttempts already exists",
            });


        const theoryTrainAttempt = await TheoryTrainAttempts.create({
            UserTheoryTrainId: userTheoryTrainId,
            endAt,
            status,
            score

        });

        if (theoryTrainAttempt) {

            return res.status(201).json({
                success: true,
                message: "New TheoryTrainAttempts has been added",
                result: theoryTrainAttempt,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
