const Sub = require("../models/Sub");
const UserSub = require("../models/UserSub");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    // [Op.or]: [{ type: { [Op.like]: whereSearch } }],
                }
                : {},
                include: [{ model: Sub }]
        };
        const info = await UserSub.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no UserSubscriptions!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All UserSubscriptions info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await UserSub.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "UserSubscriptions not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "UserSubscriptions info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateUserSubscriptions = async (req, res) => {
    try {
        const id = req.body.id;

        // const userId = req.body.userId == undefined ? undefined : req.body.userId;
        const subDate = req.body.subDate == undefined ? undefined : req.body.subDate;
        const subExpiryDate = req.body.subExpiryDate == undefined ? undefined : req.body.subExpiryDate;
        const status = req.body.status == undefined ? undefined : req.body.status;
        // const subId = req.body.subId == undefined ? undefined : req.body.subId;




        const result = await UserSub.update(
            {
                //   UserId:userId,
                //   SubId:subId,
                subDate: subDate,
                subExpiryDate: subExpiryDate,
                status: status


            },
            { where: { id: id }, individualHooks: true }
        );
        result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `UserSubscriptions is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for UserSubscriptions, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteUserSubscriptionsById = async (req, res) => {
    try {
        const id = req.query.id;
        const userSubscriptions = await UserSub.findByPk(id);

        if (userSubscriptions) {
            await userSubscriptions.destroy();

            return res.status(200).json({
                success: true,
                message: `UserSubscriptions with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "UserSubscriptions does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addUserSubscriptions = async (req, res) => {
    try {
        const { subDate, subExpiryDate, status, userId, subId } = req.body;
        const userSubscriptionsdb = await UserSub.findOne({ where: { userId, subId } });

        if (userSubscriptionsdb)
        return res.status(409).json({
            success: false,
                message: "UserSubscriptions already exists",
            });


        const UserSubscriptions = await UserSub.create({
            subDate,
            subExpiryDate,
            UserId: userId,
            SubId: subId,
            status

        });

        if (UserSubscriptions) {

            return res.status(201).json({
                success: true,
                message: "New UserSubscriptions has been added",
                result: UserSubscriptions,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};
