const Answer = require("../models/Answer");
const Choice = require("../models/Choice");
const Question = require("../models/Question");
const TheoryTrain = require("../models/TheoryTrain");
const UserTheoryTrain = require("../models/UserTheoryTrain");

const { Op } = require("sequelize");



//!  =-=-=-=-=-=-=-=-=-=  GET ALL  =-=-=-=-=-=-=-=-=-=-=//

exports.showAll = async (req, res) => {
    try {
        const whereSearch =
            req.query.search == null ? "%%" : `%${req.query.search}%`;

        const options = {
            page: req.query.page ? parseInt(req.query.page) : 1, // Default 1
            paginate: req.query.paginate ? parseInt(req.query.paginate) : 25, // Default 25
            where: req.query.search
                ? {
                    [Op.or]: [],
                }
                : {},
        };
        const info = await UserTheoryTrain.paginate(options);

        if (info == null) {
            return res.status(204).json({
                success: false,
                message: "There is no UserTheoryTrain!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "All UserTheoryTrains info",
                details: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By ID =-=-=-=-=-=-=-=-=-=-=//

exports.showById = async (req, res) => {
    try {
        const id = req.query.id;
        const info = await UserTheoryTrain.findOne({ where: { id: id } });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "UserTheoryTrain not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "UserTheoryTrain info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  UPDATE  =-=-=-=-=-=-=-=-=-=-=//

exports.updateUserTheoryTrain = async (req, res) => {
    try {
        const id = req.body.id;

        const endAt = req.body.endAt == undefined ? undefined : req.body.endAt;
        const consumedAttempts = req.body.consumedAttempts == undefined ? undefined : req.body.consumedAttempts;
        const status = req.body.status == undefined ? undefined : req.body.status;
        const userId = req.body.userId == undefined ? undefined : req.body.userId;
        const theoryTrainId = req.body.theoryTrainId == undefined ? undefined : req.body.theoryTrainId;





        const result = await UserTheoryTrain.update(
            {
                endAt,
                consumedAttempts,
                status,
                UserId: userId,
                TheoryTrainId: theoryTrainId
            },
            { where: { id: id }, individualHooks: true }
        );

        const user = result[1][0];
        if (result[0] > 0) {


            return res.status(200).json({
                success: true,
                message: `UserTheoryTrain is updated`,
            });
        } else {

            return res.status(204).json({
                success: false,
                message: `nothing updated for UserTheoryTrain, may be column not exists`,
            });
        }
    } catch (err) {

        return res.status(500).json({ success: false, message: err.message });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  DELETE  =-=-=-=-=-=-=-=-=-=-=//

exports.deleteUserTheoryTrainById = async (req, res) => {
    try {
        const id = req.query.id;
        const userTheoryTrain = await UserTheoryTrain.findByPk(id);

        if (userTheoryTrain) {
            await userTheoryTrain.destroy();

            return res.status(200).json({
                success: true,
                message: `UserTheoryTrain with id=${id} was deleted successfully!`,
            });
        } else {
            return res.status(404).json({
                success: false,
                message: "UserTheoryTrain does not exist!",
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  CREATE  =-=-=-=-=-=-=-=-=-=-=//

exports.addUserTheoryTrain = async (req, res) => {
    try {
        const { endAt, consumedAttempts, status, userId, theoryTrainId } = req.body;
        const userTheoryTraindb = await UserTheoryTrain.findOne({ where: { userId, theoryTrainId } });

        if (userTheoryTraindb)
            return res.status(409).json({
                success: false,
                message: "UserTheoryTrain already exists",
            });


        const userTheoryTrains = await UserTheoryTrain.create({
            endAt,
            consumedAttempts,
            status,
            UserId: userId,
            TheoryTrainId: theoryTrainId

        });

        if (userTheoryTrains) {

            return res.status(201).json({
                success: true,
                message: "New UserTheoryTrain has been added",
                result: userTheoryTrains,
            });
        }
    } catch (err) {

        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};

//!  =-=-=-=-=-=-=-=-=-=  GEt By UserId and theoryTrainId=-=-=-=-=-=-=-=-=-=-=//

exports.showByUserAndTheory = async (req, res) => {
    try {
        const { userId, theoryTrainId } = req.query;
        const info = await UserTheoryTrain.findOne({
            include: [
                {
                    model: Answer,
                    include: {
                        model: Choice, include: { model: Question }
                    }

                },
                {
                    model: TheoryTrain,
                    include: {
                        model: Question,
                    }

                }
            ]
            , where: { UserId: userId, TheoryTrainId: theoryTrainId }
        });

        if (info == null) {
            return res.status(404).json({
                success: false,
                message: "UserTheoryTrain not found!",
            });
        } else {
            return res.status(200).json({
                success: true,
                message: "UserTheoryTrain info",
                info: info,
            });
        }
    } catch (err) {
        return res.status(500).json({
            success: false,
            message: err.message,
        });
    }
};



