const jwt = require("jsonwebtoken");
const User = require("../models/User");
const CryptoJS = require("crypto-js");
const UserSub = require("../models/UserSub");
const Sub = require("../models/Sub");



exports.generateTokens = async (req, res, user) => {
    try {
        // const userRefreshToken = await User.findOne({where:{id:user.id}})
        
        

        // create Jwt token 
        const accessToken = jwt.sign({ user: user }, process.env.JWT_KEY, { expiresIn: process.env.REFRESH_JWT_EXPIRES_IN});
        
        // create Refresh token 
        const refreshToken = jwt.sign({ user: user }, process.env.REFRESH_TOKEN_KEY, { expiresIn:  process.env.JWT_EXPIRES_IN});
        const encryptedRefreshToken = CryptoJS.AES.encrypt(refreshToken, process.env.S_KEY).toString();


        // Update the user with the new refresh token

        const userInformation = await User.update(
            {
               hashRt:encryptedRefreshToken 
            },
            { where: { id: user.id }, individualHooks: true }
        );

        const userInfo = await User.findOne({where:{id:user.id},
        
            include: [{ model: UserSub, include: [{ model: Sub }] }]})


        // user.hashRt= refreshToken
        // await user.save()

        // Options for access token cookie
        const accessOptions = { expires: new Date(Date.now() + 24 * 60 * 60 * 1000), httpOnly: true };

        // Options for refresh token cookie
        const refreshOptions = { expires: new Date(Date.now() + 7 * 24 * 60 * 60 * 1000), httpOnly: true };

        // Set session variables if needed
        req.session.isAuth = true;
        req.session.user = user;
        req.session.userAgent = req.get('User-Agent');
        req.session.token = accessToken;
        req.session.UserId = user.id;
        req.session.refreshToken = encryptedRefreshToken;

        res.status(200)
            .header('token', accessToken)
            .cookie("token", accessToken, accessOptions)
            .cookie("refreshToken", encryptedRefreshToken, refreshOptions)
            .json({
                success: true,
                message: 'Tokens generated successfully',
                accessToken,
                userInfo,
                sessionID: req.sessionID,
            });

    } catch (error) {
        res.status(500).json({ success: false, message: error.message });
    }
};


