const router = require('express').Router()
const Controller = require("../controller/UserTheoryTrain")
const { authUser,authorizeRoles } = require('../middleware/auth');

router.get('/showall',authUser,Controller.showAll )
router.get('/',authUser,Controller.showById)
router.put('/',authUser,Controller.updateUserTheoryTrain)
router.delete('/',authUser,Controller.deleteUserTheoryTrainById)
router.post('/',authUser,Controller.addUserTheoryTrain)
router.get('/byUserbyTheory',authUser,Controller.showByUserAndTheory)
module.exports = router;

/**
 * @swagger
 * components:
 *   schemas:
 *     UserTheoryTrain:
 *       type: object
 *       properties:
 *         id:
 *           type: string
 *           description: Unique identifier (UUID) for the UserTheoryTrain
 *         consumedAttempts:
 *           type: integer
 *           description: Number of consumed attempts
 *         endAt:
 *           type: string
 *           format: time
 *           description: End time of the UserTheoryTrain
 *         status:
 *           type: boolean
 *           description: Status of the UserTheoryTrain
 *         userId:
 *           type: string
 *           description: ID of the associated User
 *         theoryTrainId:
 *           type: string
 *           description: ID of the associated TheoryTrain
 *       required:
 *         - consumedAttempts
 *         - endAt
 *         - status
 *         - userId
 *         - theoryTrainId
 *
 * tags:
 *   name: UserTheoryTrain
 *   description: Operations related to UserTheoryTrain management
 */

/**
 * @swagger
 * /usertheorytrain/showall:
 *   get:
 *     summary: Get all UserTheoryTrains
 *     tags: [UserTheoryTrain]
 *     security:
 *       - TokenAuth: []  # Requires a valid JWT token
 *     responses:
 *       '200':
 *         description: Successful operation
 *         content:
 *           application/json:
 *             schema:
 *               type: array
 *               items:
 *                 $ref: '#/components/schemas/UserTheoryTrain'
 */

/**
 * @swagger
 * /usertheorytrain:
 *   get:
 *     summary: Get a specific UserTheoryTrain by ID
 *     tags: [UserTheoryTrain]
 *     security:
 *       - TokenAuth: []  # Requires a valid JWT token
 *     parameters:
 *       - in: query
 *         name: id
 *         schema:
 *           type: string
 *         description: UserTheoryTrain ID
 *     responses:
 *       '200':
 *         description: Successful operation
 *         content:
 *           application/json:
 *             schema:
 *               $ref: '#/components/schemas/UserTheoryTrain'
 */

/**
 * @swagger
 * /usertheorytrain:
 *   put:
 *     summary: Update a specific UserTheoryTrain
 *     tags: [UserTheoryTrain]
 *     security:
 *       - TokenAuth: []  # Requires a valid JWT token
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             type: object
 *             properties:
 *               id:
 *                 type: string
 *                 description: ID of the UserTheoryTrain to update
 *               consumedAttempts:
 *                 type: integer
 *                 description: Number of consumed attempts
 *               endAt:
 *                 type: string
 *                 format: time
 *                 description: End time of the UserTheoryTrain
 *               status:
 *                 type: boolean
 *                 description: Status of the UserTheoryTrain
 *               userId:
 *                 type: string
 *                 description: ID of the associated User
 *               theoryTrainId:
 *                 type: string
 *                 description: ID of the associated TheoryTrain
 *             required:
 *               - id
 *               - consumedAttempts
 *               - endAt
 *               - status
 *               - userId
 *               - theoryTrainId
 *     responses:
 *       '200':
 *         description: Successful operation
 *         content:
 *           application/json:
 *             schema:
 *               $ref: '#/components/schemas/UserTheoryTrain'
 */

/**
 * @swagger
 * /usertheorytrain:
 *   delete:
 *     summary: Delete a specific UserTheoryTrain by ID
 *     tags: [UserTheoryTrain]
 *     security:
 *       - TokenAuth: []  # Requires a valid JWT token
 *     parameters:
 *       - in: query
 *         name: id
 *         schema:
 *           type: string
 *         description: UserTheoryTrain ID
 *     responses:
 *       '200':
 *         description: Successful operation
 *         content:
 *           application/json:
 *             schema:
 *               $ref: '#/components/schemas/UserTheoryTrain'
 */

/**
 * @swagger
 * /usertheorytrain:
 *   post:
 *     summary: Add a new UserTheoryTrain
 *     tags: [UserTheoryTrain]
 *     security:
 *       - TokenAuth: []  # Requires a valid JWT token
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             $ref: '#/components/schemas/UserTheoryTrain'
 *           example:
 *             consumedAttempts: 2
 *             endAt: "12:00:00"
 *             status: true
 *             userId: "user_id_here"
 *             theoryTrainId: "theory_train_id_here"
 *     responses:
 *       '200':
 *         description: Successful operation
 *         content:
 *           application/json:
 *             schema:
 *               $ref: '#/components/schemas/UserTheoryTrain'
 */

/**
 * @swagger
 * /usertheorytrain/byUserbyTheory:
 *   get:
 *     summary: Get UserTheoryTrain by User ID and TheoryTrain ID
 *     tags: [UserTheoryTrain]
 *     security:
 *       - TokenAuth: []  # Requires a valid JWT token
 *     parameters:
 *       - in: query
 *         name: userId
 *         schema:
 *           type: string
 *         description: User ID to filter UserTheoryTrain
 *       - in: query
 *         name: theoryTrainId
 *         schema:
 *           type: string
 *         description: TheoryTrain ID to filter UserTheoryTrain
 *     responses:
 *       '200':
 *         description: Successful operation
 *         content:
 *           application/json:
 *             schema:
 *               $ref: '#/components/schemas/UserTheoryTrain'
 */
