const express = require("express");
require("dotenv").config();
var cors = require("cors");
var app = express();
const swaggerJsdoc = require('swagger-jsdoc');
const swaggerUi = require('swagger-ui-express');
// const path = require("path");
require('./api/models/Session')
const SequelizeStore = require("connect-session-sequelize")(require("express-session").Store);
const sequelize = require('./config/database');
const constants = require('./config/constants.json');



app.use(cors({ origin: "*", }));
app.use(express.json({ limit: "100mb", }));
app.use(express.urlencoded({ extended: true }));



function extendDefaultFields(defaults, session) {
  return {
    data: defaults.data,
    expires: defaults.expires,
    sid: session.sid,
    isAuth: session.isAuth,
    user: session.user,
    userAgent: session.userAgent,
    UserId: session.UserId
  };
}
const options = {
  definition: {
    openapi: '3.0.0',
    info: {
      title: 'Driver Test API',
      version: '1.0.0',
    },

    components: {
      securitySchemes: {
        TokenAuth: {
          type: "http",
          scheme: "bearer",
          bearerFormat: "Token",
        },
      },
    },
    security: [

      {
        TokenAuth: [],
      },
    ],
    servers: [
      {
        // todo Basic Url
        url: process.env.BASE_URL || 'http://localhost:8000/api',
        //  http://localhost:8000/api-docs/#/
      },
    ],
  },
  apis: ['./api/routes/*.js'], // Specify the path to your route files
};

const specs = swaggerJsdoc(options);
app.use('/api-docs', swaggerUi.serve, swaggerUi.setup(specs));


var store = new SequelizeStore({
  db: sequelize,
  table: "Session",
  extendDefaultFields: extendDefaultFields,
});


const session = require("express-session")({
  secret: process.env.S_KEY,
  store: store,
  saveUninitialized: true,
  resave: false, // we support the touch method so per the express-session docs this should be set to false
  proxy: true, // if you do SSL outside of node.
}
)



var compression = require('compression')

app.use(compression())

// Attach session
app.use(session);

// Share session with io sockets





const faq = require("./api/routes/Faq");
const user = require("./api/routes/User");
const blog = require("./api/routes/Blog");
const school = require("./api/routes/School")
const sub = require("./api/routes/Sub")
const usersub = require("./api/routes/UserSub")
const pracList = require("./api/routes/PracTrainPlayList")
const pracVideo = require("./api/routes/PracTrainVideo")
const prevVideo = require("./api/routes/PreviewVideo")
const theoryTrain = require("./api/routes/TheoryTrain")
const question = require("./api/routes/Question")
const choice = require("./api/routes/Choices")
const answer = require("./api/routes/Answer")
const usertheorytrain = require("./api/routes/UserTheoryTrain")
const theorytrainattempts = require("./api/routes/TheoryTrainAttempts")
const exam = require("./api/routes/Exam")





app.use("/api/blog", blog);
app.use("/api/user", user);
app.use("/api/faq", faq);
app.use("/api/school", school);
app.use("/api/subs", sub);
app.use("/api/usersubs", usersub);
app.use("/api/pracLists", pracList);
app.use("/api/pracvideos", pracVideo);
app.use("/api/previewvideos", prevVideo);
app.use("/api/theorytrains", theoryTrain);
app.use("/api/questions", question);
app.use("/api/choice", choice);
app.use("/api/answer", answer);
app.use("/api/usertheorytrain", usertheorytrain);
app.use("/api/theorytrainattempts", theorytrainattempts);
app.use("/api/exam", exam);






app.use('/api/blogs', express.static(constants.assets_loc.blogs))
app.use('/api/questions', express.static(constants.assets_loc.questions))
app.use('/api/users', express.static(constants.assets_loc.users))



// app.use('/api/companies', express.static(constants.assets_loc.companies))


// //* Have Node serve the files for our built React app
// app.use(express.static(path.resolve(__dirname, './build')));

// //* All other GET requests not handled before will return our React app
// app.get('*', (req, res) => {
//   res.sendFile(path.resolve(__dirname, './build', 'index.html'));
// });

//? ================================================
// !! =========== ERROR Handling URL ========= !! //
//? ================================================

app.use((req, res, next) => {
  const error = new Error("Route Not Found!");
  error.status = 404;
  next(error);
});

app.use((error, req, res, next) => {
  res.status(error.status || 500);
  res.json({
    error: {
      message: error.message,
    },
  });
});

module.exports = app;
